"Script for encrypting/decrypting files and folders."

import os
from log import Log
from Crypto.Cipher import AES
from Crypto.Protocol.KDF import PBKDF2


class Cipher:
    """
    A class for ciphering/deciphering operations.

    Linux native.

    Parameters:
        Username: The Linux username of the user who is logging in.
        Password: The password of the user.
    """


    def __init__(self, password=str, username=str) -> None:
        self.password = password
        self.username = username
        self.marker  = b"E1m%nj2i$bhilj"
        self.logger = Log(self.username)


    def log(self, mode=str, path=str, encrypted=bool) -> int:
        self.logger.log(mode=mode, path=path, encrypted=encrypted)
        pass


    def guard_clause(self, path=str or None, decrypting=bool) -> int:
        """
        A guard clause for file operations.

        Return codes:
            0: File path is good.
            1: File path is None.
            2: File path is invalid.
            3: App does not have permission to access file.
            4: File is hidden.
            5: File is part of root filesystem and not accessing chromeos.
            6: File encryption marker detected during encryption.
            7: File encryption marker not detected during decryption.
        """
        if path is None:
            return 1
        # checking if file is hidden
        if "/." in path:
            return 4
        # checking if file is part of the root filesystem and not a part of ChromeOS for linux in Chromebooks
        if "/home/" not in path and "/mnt/chromeos/MyFiles" not in path:
            return 5
        # checking if decrypt is cancelleD
        try:
            if os.path.isfile(path) is True:
                with open (path, "rb") as file:
                    file_marker = file.read(14)
            elif os.path.isdir(path) is True:
                return 0
        # checking if file exists
        except FileNotFoundError:
            return 2
        # checking if application has permission
        except PermissionError:
            return 3
        # checking if there is a file encryption marker during encryption
        if file_marker == self.marker and decrypting is False:
            return 6
        # checking if there is not a file encryption marker during decryption
        elif file_marker != self.marker and decrypting is True:
            return 7
        else:
            # deleting everything
            del file_marker
            return 0


    def encrypt(self, path=str) -> int:
        """
        A method for encrypting a file/folder object.

        Takes only a file path as an argument.

        Return codes:
            0: File encrypt successful.
            1: Path is None.
            2: Path is invalid.
            3: App does not have permissions required to access file.
            4: File is hidden.
            5: File is part of root filesystem and not part of ChromeOS.
            6: File encryption marker detected during encryption.
        """
        return_code = self.guard_clause(path, decrypting=False)
        if return_code != 0:
            return return_code
        salt = os.urandom(32)
        iv = os.urandom(16)
        key = PBKDF2(self.password, salt, dkLen=32)
        key_hash = hash.sha256(key); key_signature = key_hash.digest()
        if os.path.isfile(path) is True:
            with open (path, "rb") as file:
                data = file.read()
            cipher = AES.new(key, AES.MODE_CFB, iv=iv)
            ciphered_data = cipher.encrypt(data)
            with open(path, "wb") as file:
                file.write(self.marker)
                file.write(iv)
                file.write(salt)
                file.write(key_signature)
                file.write(ciphered_data)
            objects = [return_code, salt, iv, key, key_hash, key_signature, data, cipher, ciphered_data]
            for object_ in objects:
                del object_
            self.log("s", path, encrypted=True)
            return 0
        elif os.path.isdir(path) is True:
            for root, dirs, files in os.walk(path):
                for file in files:
                    file_name = os.path.join(root, file)
                    return_code = self.guard_clause(file_name, decrypting=False)
                    if return_code != 0:
                        continue
                    with open (file_name, "rb") as file:
                        data = file.read()
                    cipher = AES.new(key, AES.MODE_CFB, iv=iv)
                    ciphered_data = cipher.encrypt(data)
                    with open(file_name, "wb") as file:
                        file.write(self.marker)
                        file.write(iv)
                        file.write(salt)
                        file.write(key_signature)
                        file.write(ciphered_data)
                    objects = [return_code, salt, iv, key, key_hash, key_signature, data, cipher, ciphered_data]
                    for object_ in objects:
                        del object_
            self.log("f", path, encrypted=True)
            return 0


    def decrypt(self, path=str) -> int:
        """
        A method for decrypting an encrypted file.

        Takes only a file path as an argument.

        return codes:
            0: File decrypt successful.
            1: Path is None.
            2: File path is invalid.
            3: App does not have permissions required to access file.
            4: File is hidden.
            5: File is part of root filesystem and not part of ChromeOS.
            6: File was encrypted with a different key.
            7: File was not encrypted.
        """
        return_code = self.guard_clause(path, decrypting=True)
        if return_code != 0:
            return return_code
        if os.path.isfile(path) is True:
            with open (path, "rb") as file:
                file.read(14)
                iv = file.read(16)
                salt = file.read(32)
                file_signature = file.read(32)
                ciphered_data = file.read()
            key = PBKDF2(self.password, salt, dkLen=32)
            key_hash = hash.sha256(key); key_signature = key_hash.digest()
            if file_signature != key_signature:
                objects = [return_code, iv, salt, file_signature, ciphered_data, key, key_hash, key_signature]
                for object_ in objects:
                    del object_
                return 6
            cipher = AES.new(key, AES.MODE_CFB, iv=iv)
            original_data = cipher.decrypt(ciphered_data)
            with open(path, "wb") as file:
                file.write(original_data)
            objects = [return_code, iv, salt, file_signature, ciphered_data, key, key_hash, key_signature, cipher, original_data]
            for object_ in objects:
                del object_
            self.log("s", path, encrypted=False)
            return 0
        elif os.path.isdir(path) is True:
            for root, dirs, files in os.walk(path):
                for file in files:
                    file_name = os.path.join(root, file)
                    return_code = self.guard_clause(file_name, decrypting=True)
                    if return_code != 0:
                        continue
                    with open (file_name, "rb") as file:
                        file.read(14)
                        iv = file.read(16)
                        salt = file.read(32)
                        file_signature = file.read(32)
                        ciphered_data = file.read()
                    key = PBKDF2(self.password, salt, dkLen=32)
                    key_hash = hash.sha256(key); key_signature = key_hash.digest()
                    if key_signature != file_signature:
                        return 6
                    cipher = AES.new(key, AES.MODE_CFB, iv=iv)
                    original_data = cipher.decrypt(ciphered_data)
                    with open (file_name, "wb") as file:
                        file.write(original_data)
                    objects = [return_code, iv, salt, file_signature, ciphered_data, key, key_hash, key_signature, cipher, original_data]
                    for object_ in objects:
                        del object_
            self.log("f", path, encrypted=False)
            return 0
        

        self.log()
        pass


    def change_password(self, new_password = str):
        self.password = new_password
        pass